/*
 * Nomad Instrument Control Software
 *
 * Copyright 2011 Institut Laue-Langevin
 *
 * Licensed under the EUPL, Version 1.1 only (the "License");
 * You may not use this work except in compliance with the Licence.
 * You may obtain a copy of the Licence at:
 *
 * http://joinup.ec.europa.eu/software/page/eupl
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the Licence is distributed on an "AS IS" basis,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the Licence for the specific language governing permissions and
 * limitations under the Licence.
 */

#include "controllers/common/family/Families.h"
#include "CyclopsDataPath.h"
#include <boost/filesystem/operations.hpp>
using namespace std;

namespace cyclops {

const string CyclopsDataPath::TYPE = "cyclops_datapath";

/*
 * Constructor
 */
CyclopsDataPath::CyclopsDataPath(const string& name) :
		ExperimentController(name), controller::Start(this) {

	setFamily(family::ACQUISITION, family::SETTING);

	datapath.init(this, SAVE, "datapath");
	dataname.init(this, SAVE, "dataname");

	registerFunction(TYPE);

	m_CamDriver.init(this, "cammeraDriver");
	m_Title.init(this, "title");
}

/*
 * Destructor
 */
CyclopsDataPath::~CyclopsDataPath() {
}

/*
 * postConfiguration
 */
void CyclopsDataPath::postConfiguration() {
	registerUpdater(m_Title->numor, &CyclopsDataPath::updateNumor, this);
	updateNumor();
}

/*
 * start
 */
void CyclopsDataPath::start() {
	commandProgression = 0;
	commandStatus.setRunning();
	// Create path in /users/data if not existed
	boost::filesystem::path path = "/users/data";
	path /= datapath();
	if (boost::filesystem::is_regular_file(path)) {
		log(Level::s_Error) << "error path name existed as regular file " << path.string() << endlog;
		commandStatus.setError();
	}
	else {
		if (boost::filesystem::is_directory(path) == false) {
			boost::filesystem::create_directory(path);
			log(Level::s_Info) << "create local data path " << path.string() << endlog;
		}
		m_CamDriver->fileDirectory = datapath();
		m_CamDriver->fileName = dataname();
		log(Level::s_Info) << "set data path " << datapath() << " and name " << dataname() << endlog;
		commandStatus.setIdle();
	}
	commandProgression = 0;
}

/*
 * updateNumor
 */
void CyclopsDataPath::updateNumor() {
	m_CamDriver->numor = m_Title->numor();
}

/*
 * test
 */
void CyclopsDataPath::test() {
	using namespace utilities;
	UnitTest test(this);

	boost::filesystem::path tmp = "/users/data/CyclopsDataPathTest";
	ofstream ofs;
	ofs.open(tmp.string().c_str());
	if (ofs.is_open()) {
		ofs << "TEST" << endl;
		ofs.close();
	}
	datapath = "CyclopsDataPathTest";
	dataname = "nomad";
	this->startCommand();
	test.checkTrue(commandStatus.isError());

	boost::filesystem::remove(tmp);

	this->startCommand();
	test.checkTrue(commandStatus.isIdle());
	test.checkTrue(boost::filesystem::is_directory(tmp));

	this->startCommand();
	test.checkTrue(commandStatus.isIdle());

	boost::filesystem::remove(tmp);

}
}
