/*
 * Nomad Instrument Control Software
 *
 * Copyright 2011 Institut Laue-Langevin
 *
 * Licensed under the EUPL, Version 1.1 only (the "License");
 * You may not use this work except in compliance with the Licence.
 * You may obtain a copy of the Licence at:
 *
 * http://joinup.ec.europa.eu/software/page/eupl
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the Licence is distributed on an "AS IS" basis,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the Licence for the specific language governing permissions and
 * limitations under the Licence.
 */

#ifndef REMOTELSSSIMULATION_H
#define REMOTELSSSIMULATION_H

#include <Controller.h>
#include <memory>
#include <cameo/cameo.h>
#include "drivers/utilities/null/NullDriver.h"
#include "controllers/common/utilities/CountSpy.h"
#include "RemoteLSSSimulation.pb.h"

namespace remotelsssimulation {

class RemoteLSSSimulation: public ExperimentController, public controller::Start, public controller::Init {

public:

	//! Type of controller
	static const std::string TYPE;

	RemoteLSSSimulation(const std::string& name);
	RemoteLSSSimulation(const RemoteLSSSimulation& controller);
	virtual ~RemoteLSSSimulation();

	virtual void postConfiguration();

	Property<std::string> serverEndpoint;
	Property<bool> initialized;
	Property<int32> applicationState;

	Property<bool> useSimulatedData;

	DynamicProperty<std::string> model_name;
	DynamicProperty<std::string> model_type;
	DynamicProperty<int32> model_number;
	DynamicProperty2<std::string> pnames;
	DynamicProperty2<std::string> structname;
	DynamicProperty<std::string> fn_eval;
	DynamicProperty2<float64> parameters;

	Property<int32> nbSampleModels;
	DynamicProperty<std::string> sampleModelName;
	Property<std::string> displaySampleModelName;
	Property<std::string> displaySampleModelType;
	Property<int32> displaySampleNbParameters;
	DynamicProperty<std::string> displaySampleParametersNames;
	DynamicProperty<float64> displaySampleParametersValues;

	Property<int32> nbBackgroundModels;
	DynamicProperty<std::string> backgroundModelName;
	Property<std::string> displayBackgroundModelName;
	Property<std::string> displayBackgroundModelType;
	Property<int32> displayBackgroundNbParameters;
	DynamicProperty<std::string> displayBackgroundParametersNames;
	DynamicProperty<float64> displayBackgroundParametersValues;

	Property<int32> nbBlockedBeamModels;
	DynamicProperty<std::string> blockedBeamModelName;
	Property<std::string> displayBlockedBeamModelName;
	Property<std::string> displayBlockedBeamModelType;
	Property<int32> displayBlockedBeamNbParameters;
	DynamicProperty<std::string> displayBlockedBeamParametersNames;
	DynamicProperty<float64> displayBlockedBeamParametersValues;

private:

	virtual void start();
	virtual void init();

	bool initApplication();
	void stopApplication();
	void test();

	void getData();

	void resizeAll(int32 size);

	void refreshDisplaySampleModelProperty(const std::string& model) throw(CannotSetValue);
	void refreshDisplaySampleParametersValuesProperty(int32 i, float64 value) throw(CannotSetValue);
	void refreshDisplayBackgroundModelProperty(const std::string& model) throw(CannotSetValue);
	void refreshDisplayBackgroundParametersValuesProperty(int32 i, float64 value) throw(CannotSetValue);
	void refreshDisplayBlockedBeamModelProperty(const std::string& model) throw(CannotSetValue);
	void refreshDisplayBlockedBeamParametersValuesProperty(int32 i, float64 value) throw(CannotSetValue);

	void refreshUseSimulatedDataProperty(bool value) throw(CannotSetValue);

	void updateCountStatusMessage();
	void updateDataFromSimulation();
	void updateCountDetSum();

	static const std::string MATLAB_APPLICATION;
	static const std::string RESPONDER_NAME;

	static const std::string SAMPLE_TYPE;
	static const std::string BACKGROUND_TYPE;
	static const std::string BLOCKEDBEAM_TYPE;

	static const std::string D11_COLLIMATION_TYPE;
	static const std::string D22_COLLIMATION_TYPE;
	static const std::string D33_COLLIMATION_TYPE;

	std::unique_ptr<cameo::Server> m_server;
	std::unique_ptr<cameo::application::Instance> m_matlabApplication;
	std::unique_ptr<cameo::application::Requester> m_requester;

	DriverPtr<driver::NullDriver> m_driver;
	utilities::CountSpy* m_CountSpy;

	DataResponse m_DataResponse;
	int32 m_displaySampleModel;
	int32 m_displayBackgroundModel;
	int32 m_displayBlockedBeamModel;
};

}

#endif
