/*
 * Nomad Instrument Control Software
 *
 * Copyright 2011 Institut Laue-Langevin
 *
 * Licensed under the EUPL, Version 1.1 only (the "License");
 * You may not use this work except in compliance with the Licence.
 * You may obtain a copy of the Licence at:
 *
 * http://joinup.ec.europa.eu/software/page/eupl
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the Licence is distributed on an "AS IS" basis,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the Licence for the specific language governing permissions and
 * limitations under the Licence.
 */

#ifndef REMOTENANOKIN_H
#define REMOTENANOKIN_H

#include <Controller.h>
#include <cameo/cameo.h>
#include "controllers/common/acquisition/ExperimentData.h"
#include "drivers/utilities/null/NullDriver.h"

namespace nanokin {

class RemoteNanokin: public ExperimentController, public controller::Stoppable {

public:
	//! Type of controller
	static const std::string TYPE;

	RemoteNanokin(const std::string& name);
	RemoteNanokin(const RemoteNanokin& nano);

	virtual ~RemoteNanokin();

	virtual void postConfiguration();

	Property<std::string> serverEndpoint;
	Property<std::string> action;
	Property<std::string> sopxFile;

private:

	virtual void start();
	virtual void stop();

	void NanokinStartOutput();

	ControllerPtr<acquisition::ExperimentData> m_Title;
	DriverPtr<driver::NullDriver> m_driver;

	std::unique_ptr<cameo::Server> m_server;
	std::unique_ptr<cameo::application::Instance> m_StartApplication;
	std::unique_ptr<cameo::application::Instance> m_StopApplication;
	std::unique_ptr<cameo::application::Instance> m_FileTransfert;
	std::unique_ptr<cameo::application::Requester> m_FileRequester;
	std::shared_ptr<cameo::OutputStreamSocket> m_StartOutputSocket;

	bool m_initialized;

	static const std::string NANOKIN_START_CMD;
	static const std::string NANOKIN_STOP_CMD;
	static const std::string NANOKIN_FILETRANSFERT_CMD;

	static const std::string NANOKIN_STARTED_MSG;

	static const std::string START_ACTION;
	static const std::string END_ACTION;

};

}

#endif
