/*
 * Nomad Instrument Control Software
 *
 * Copyright 2011 Institut Laue-Langevin
 *
 * Licensed under the EUPL, Version 1.1 only (the "License");
 * You may not use this work except in compliance with the Licence.
 * You may obtain a copy of the Licence at:
 *
 * http://joinup.ec.europa.eu/software/page/eupl
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the Licence is distributed on an "AS IS" basis,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the Licence for the specific language governing permissions and
 * limitations under the Licence.
 */

#ifndef TAS_REMOTEMATLAB_H
#define TAS_REMOTEMATLAB_H

#include "controllers/common/utilities/imageviewer/ImageData.h"
#include "controllers/common/acquisition/ImageGeneratable.h"
#include "controllers/common/acquisition/ExperimentData.h"
#include "controllers/common/scanlegacy/GenericScan1D.h"
#include "controllers/common/utilities/CountSpy.h"
#include "controllers/tas/common/TasSettings.h"
#include "NumorMessages.pb.h"
#include <cameo/cameo.h>

namespace tas {

class RemoteMatlab : public utilities::ImageData,
						public acquisition::ImageGeneratable,
						public controller::Start {

public:
	//! Type of controller
	static const std::string TYPE;

	RemoteMatlab(const std::string& name);
	virtual ~RemoteMatlab();

	virtual void postConfiguration();

	void updateSettingsMode();

	Property<std::string> serverEndpoint;
	Property<bool> initialized;
	Property<bool> test;
	Property<int32> testNumor;
	Property<bool> active;

	ControllerPtr<acquisition::ExperimentData> experimentData;
	ControllerPtr<scan::GenericScan1D> scanController;
	ControllerPtr<utilities::CountSpy> countSpy;
	ControllerPtr<tas::TasSettings> tasSettings;

protected:
	std::string getProposalId() const;
	std::string getProposalName() const;
	std::string getInstrumentId() const;
	std::string getInstrumentName() const;
	std::string getSampleId() const;
	std::string getSampleName() const;
	std::string getCycleId() const;
};

}

#endif
