/*
 * Nomad Instrument Control Software
 *
 * Copyright 2011 Institut Laue-Langevin
 *
 * Licensed under the EUPL, Version 1.1 only (the "License");
 * You may not use this work except in compliance with the Licence.
 * You may obtain a copy of the Licence at:
 *
 * http://joinup.ec.europa.eu/software/page/eupl
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the Licence is distributed on an "AS IS" basis,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the Licence for the specific language governing permissions and
 * limitations under the Licence.
 */

#include "RealIKonDriver.h"

#include <cstdio>
#include "IKonDriver.h"
#include "atmcdLXd.h"


#include "Utilities/Counter.h"

using namespace std;

namespace ikon {

/*
 * Constructor
 */
RealIKonDriver::RealIKonDriver(IKonDriver* owner) :
																										IKonState(owner) {
	/* Empty */
}

/*
 * Destructor
 */
RealIKonDriver::~RealIKonDriver() {
	ShutDown();
}

/*
 * init
 */
void RealIKonDriver::init() {
	uint32 error;
	bool quit;
	char choice;
	float fChoice;
	int width, height;
	at_32 lNumCameras;

	GetAvailableCameras(&lNumCameras);
	if (lNumCameras<0) {
		cout << "*** CAMERA SELECTION ERROR" << endl;

	}else{


		//Initialize CCD
		error = Initialize("/usr/local/etc/andor");
		if(error!=DRV_SUCCESS){
			cout << "Initialisation error...exiting" << error<<endl;

		} else {

			sleep(2); //sleep to allow initialization to complete
			owner()->wantedTemperature=-80;
			SetTemperature(-80);
			CoolerON();
			//Set Read Mode to --Image--
			SetReadMode(AC_READMODE_SINGLETRACK);
			// Full SetReadMode(0);
			//Set Acquisition mode to --Single scan--
			SetAcquisitionMode(AC_ACQMODE_SINGLE);
			owner()->acqmode=AC_ACQMODE_SINGLE;
			owner()->time.setpoint=DEFAULT_TIME;
			owner()->readout=AC_READMODE_SINGLETRACK;

			// Full  owner()->readout=0;
			//Set initial exposure time
			SetExposureTime(DEFAULT_TIME);
			//Initialize Shutter
			SetShutter(1,0,50,50);

			// Set Horizontal Speed to 3MHz
			SetHSSpeed(0,1);

			// Set active
			SetBaselineClamp(1);

		}

		read();
		readInfos();
		readStatus();
	}

	owner()->acqmode = 3;
	writeParam();
	owner()->acqmode = 1;
	writeParam();


}

/*
 * clear
 */
void RealIKonDriver::clear() {
	memset(owner()->data(), 0, owner()->dataSize() * sizeof(int32));
}

/*
 * writeParam
 */
void RealIKonDriver::writeParam() {
	int32 error;
	int rc;

	 Counter c1;

	rc =SetTemperature(owner()->wantedTemperature());
	if( rc != DRV_SUCCESS )
	{
		cout << "SetTemperature() error" << endl;
	}else{
		cout << "SetTemperature() to " <<owner()->wantedTemperature()<< endl;

	}

	 cout << " writeParam0 SetTemperature " << c1.getTime() << endl;c1.clear();

	//Set initial exposure time
	rc =SetExposureTime( owner()->time.setpoint());
	if( rc != DRV_SUCCESS )
	{
		cout << "SetExposureTime() error" << endl;
	}

	 cout << " writeParam0  SetExposureTime " << c1.getTime() << endl;c1.clear();


	owner()->width=DEFAULT_WIDTH;
	owner()->height=DEFAULT_HEIGHT;
	//rc =SetReadMode(0);
	rc =SetReadMode(owner()->readout());
	if( rc != DRV_SUCCESS )
	{
		cout << "SetReadMode() error" << endl;
	}

	 cout << " writeParam0 SetReadMode " << c1.getTime() << endl;c1.clear();

	//Setup Image dimensions

	if (owner()->modeFVB() == false) {
		SetAcquisitionMode(owner()->acqmode());
		 cout << " writeParam0 SetImage owner()->modeFVB() == false "<< endl;
		rc = SetImage(owner()->hbind(), owner()->vbind(), 1, owner()->width(),1, owner()->height());
	} else {

		 cout << " writeParam0 SetImage owner()->modeFVB() == true "<< endl;
		 rc = SetImage(1, 1, 1, owner()->width(), 1, 1);
	}

	// Sensor not oriented  we switch X and Y
	//rc =SetImage(owner()->vbind(),owner()->hbind(),1,owner()->width(),1,owner()->height());
	 // Full  rc =SetImage(owner()->vbind(),1,1,owner()->width(),1,1);


	// Test speed BOB
	//rc =SetImage(100,1,  50,150,  1,2048);
	// rc =SetImage(1,100,  1, 2048,    50,150);
	// rc =SetImage(1,1,  1, 10,    1,10);

	 cout << " writeParam0  SetImage" << c1.getTime() << endl;c1.clear();



	if( rc != DRV_SUCCESS )
	{
		cout << "SetImage() error" << endl;
	}

// Dont send 	SetImageFlip SetImageRotate
//	rc =SetImageFlip( owner()->hflip(),owner()->vflip());
//	if( rc != DRV_SUCCESS )
//	{
//		cout << "SetImageFlip() error" << endl;
//	}
//	rc =SetImageRotate(owner()->rotate());
//	if( rc != DRV_SUCCESS )
//	{
//		cout << "SetImageRotate() error" << endl;
//	}


	rc =SetShutter(1,owner()->shutmode(),owner()->shutclosetime(),owner()->shutopentime());
	if( rc != DRV_SUCCESS )
	{
		cout << "SetShutter() error" << endl;
	}

	rc =SetBaselineClamp(owner()->bclamp());

	if( rc != DRV_SUCCESS )
	{
		cout << "SetBaselineClamp() error" << endl;
	}




	rc =SetAcquisitionMode(owner()->acqmode());

	if( rc != DRV_SUCCESS )
	{
		cout << "SetAcquisitionMode() error" << endl;
	}


	rc = SetHSSpeed( 0, owner()->hsspeed() );
	if( rc != DRV_SUCCESS )
	{
		cout << "SetHSSpeed() error" << endl;
	}

	rc = SetPreAmpGain( owner()->gain()  );
	if( rc != DRV_SUCCESS )
	{
		cout << "SetPreAmpGain() error" << endl;

	}




	if (owner()->acqmode()==3){
		rc = SetKineticCycleTime(owner()->time.setpoint() + 0.001);
		if( rc != DRV_SUCCESS )
		{
			cout << "SetKineticCycleTime() error" << endl;

		}
		rc = SetNumberKinetics(owner()->nbSlicesAcq());
		if( rc != DRV_SUCCESS )
		{
			cout << "SetKineticCycleTime() error" << endl;

		}
	}

	 cout << " writeParam0 other param " << c1.getTime() << endl;c1.clear();

}

/*
 * synchroniseRead
 */
void RealIKonDriver::synchroniseRead() {
	uint32 size;

	cout << "RealIKonDriver synchroniseRead synchroniseRead synchroniseRead " << endl;

	 Counter c1;

	/*
	 * Calculate the detector sum here
	 */
	int32* data = owner()->data();
	int32* mask = owner()->mask();
	int32 dataSize = owner()->dataSize();
	float64 sum = 0;
	if (data != NULL) {
		for (int32 i = 0; i < dataSize; ++i) {
			sum += (float64) data[i];
		}
	} else {
		cerr << "error mask " << endl;

	}
	owner()->sum = sum;
	owner()->data.sendEvent();

	if (owner()->modeFVB() == false) {
		size = owner()->width() * owner()->height();
		size = size / (owner()->hbind() * owner()->vbind());
	} else {
		size = DEFAULT_WIDTH;
	}

	if (owner()->acqmode()==3){
		size=size*owner()->nbSlicesAcq();
	}

	// Test speed BOB
	//size= 2088;


	/* Finish the sequence */
	cerr << "synchroniseReadRead01 size " << size << endl;
	GetAcquiredData(data,size);
	owner()->m_SynchroniseReadTerminated = true;


	cout << "RealIKonDriver read Data  " << c1.getTime() << endl;c1.clear();
}

/*
 * read
 */
void RealIKonDriver::read() {
	int32 temp;
	int32 ret= GetTemperature(&temp);
	owner()->actualTemperature=temp;
	owner()->tempstatus=ret;

}

/*
 * start
 */
void RealIKonDriver::start() {

	 Counter c1;


	uint32 size;
	int32 status;
	int32 error;
	float64 exp_time =  owner()->time.setpoint();
	clear();

	// Full SetReadMode(0);
				//Set Acquisition mode to --Single scan--
				//SetAcquisitionMode(AC_ACQMODE_SINGLE);


	if (owner()->modeFVB() == false) {
		size = owner()->width() * owner()->height();
		size = size / (owner()->hbind() * owner()->vbind());
		cout <<"owner()->modeFVB() == false  acqmode " << owner()->acqmode() << endl;
		SetAcquisitionMode(owner()->acqmode());
	} else {
		size = DEFAULT_WIDTH;
		cout <<"owner()->modeFVB() == true  acqmode 0  "  << endl;
		SetReadMode(0);
	}

	if (owner()->acqmode()==3){
		exp_time=owner()->time.setpoint();
		size=size*owner()->nbSlicesAcq();
	}
	int32* data = owner()->data();
	SetAcquisitionMode(owner()->acqmode());
	SetExposureTime(exp_time);
	StartAcquisition();




	cout << "RealIKonDriver start01   " << c1.getTime() << endl;


	at_32 acc, kin;
	int stat;

	do{
		unsigned int ret = WaitForAcquisition();
		cout << "RealIKonDriver start0100   " << c1.getTime() << endl;
		if(ret==DRV_NO_NEW_DATA) {
			AbortAcquisition();
			cout << "RealIKonDriver start0101   " << c1.getTime() << endl;
			cout << "DRV_NO_NEW_DATA" << endl;
			owner()->startActivated = false;
			owner()->commandProgression = PROGRESSION_END_DEVICE_CONTAINER;
			break;
		}
//		usleep(10000);

		GetAcquisitionProgress(&acc, &kin);
		cout << "RealIKonDriver start0102   " << c1.getTime() << endl;
		cout << "Accums=" << acc << " Kinetics=" << kin << endl;
		GetStatus(&stat);
		cout << "RealIKonDriver start0103   " << c1.getTime()  << "Status " <<  stat << endl;

	}while(stat==DRV_ACQUIRING);

	cout << "RealIKonDriver start02   " << c1.getTime() << endl;

	cout <<  " fin acc GetAcquiredData size " <<  size<< endl;

	//BOB
	//size= 2088;
	error=GetAcquiredData(data,size);
	if (error==DRV_SUCCESS) {
		m_TimeCounter.clear();
	} else {
		cerr << "error acquisition1 " <<error<< endl;
	}
	int32 dataSize = owner()->dataSize();
	cout << "dataSize " << dataSize << endl;




//	if (data != NULL) {
//		for (int32 i = 0; i < dataSize; ++i) {
//			data[i] = data[i]-300;
//		}
//	} else {
//		cerr << "error mask " << endl;
//
//	}

	//
	//	at_32* imageData = new at_32[size];
	//	error=GetAcquiredData(imageData,size);
	//	if (error==DRV_SUCCESS) {
	//		m_TimeCounter.clear();
	//	} else {
	//cerr << "fin acquisition22 " <<error<< endl;
	//	}



}


/*
 * resume
 */
void RealIKonDriver::resume() {
}

/*
 * pause
 */
void RealIKonDriver::pause() {

}

/*
 * stop
 */
void RealIKonDriver::stop() {
	AbortAcquisition();
	owner()->startActivated = false;

	owner()->commandProgression = PROGRESSION_END_DEVICE_CONTAINER;
}

/*
 * readStatus
 */
void RealIKonDriver::readStatus() {

	/*
	 * Read the status card
	 */
	int32 value;
	unsigned long not_needed;
	GetStatus(&value);
	// Status command
	owner()->computeGateStatus(value);
	owner()->computeAcquisitionStatus(0);
	owner()->computeDetectorStatus(0);




	if (owner()->startActivated() == true) {
		if ((value == DRV_IDLE) || (value ==DRV_DRIVER_ERRORS )) {
			if (value == DRV_DRIVER_ERRORS) {
				cerr << "Data collection error: " << value << endl;
			}
			AbortAcquisition();
			owner()->startActivated = false;
			//synchroniseRead();
			owner()->time = owner()->time.setpoint();
			owner()->commandProgression = PROGRESSION_END_DEVICE_CONTAINER;
		} else {
			float64 aTime = m_TimeCounter.getTime();
			if (aTime <= owner()->time.setpoint()) {
				owner()->time = aTime;
			}
			owner()->calculateProgression();
		}
	} else {
		if (owner()->m_SynchroniseReadActivated == true) {
			if (owner()->m_SynchroniseReadTerminated == true) {
				owner()->m_SynchroniseReadActivated = false;
				owner()->m_SynchroniseReadTerminated = false;
				owner()->commandProgression = PROGRESSION_END_DEVICE_CONTAINER;
				owner()->time = owner()->time.setpoint();
			}
		} else {
			owner()->commandProgression = PROGRESSION_END_DEVICE_CONTAINER;
			owner()->time = owner()->time.setpoint();
		}
	}
	//	}

}

/*
 * readInfos
 */
void RealIKonDriver::readInfos() {
	int32 index=0;
	int32  information;
	string info;
	char car[260];   // per manual


	int num_channels, num_amps, num_preamp_gains;
	int num_HSS ;
	int ch, bits, amp, hhs, i,g;
	float gain;
	float f;

	GetNumberADChannels( &num_channels );
	printf("GetNumberADChannels() is %d\n", num_channels);
	GetNumberAmp( &num_amps );
	printf("GetNumberAmp() is %d\n", num_amps);
	GetNumberPreAmpGains( &num_preamp_gains );
	printf("GetNumberPreAmpGains() is %d\n", num_preamp_gains);

	for( ch=0; ch<num_channels; ch++ )
	{
		GetBitDepth( ch, &bits);
		for( amp=0; amp<num_amps; amp++ )
		{
			GetNumberHSSpeeds( ch, amp, &num_HSS );
			for(hhs=0; hhs<num_HSS; hhs++ )
			{
				GetHSSpeed( ch, amp, hhs, &f);
				printf( "ch=%d bits=%d amp=%d HSS=%d is %f MHz \n", ch, bits, amp, hhs, f);
				for( g=0; g<num_preamp_gains; g++)
				{
					GetPreAmpGain( g, &gain);
					IsPreAmpGainAvailable( ch, amp, hhs, g, &i);
					printf("   g=%d gain=%f OK=%d \n", g, gain, i);
				}
			}
		}
	}





	GetCameraSerialNumber(&information);
	ostringstream buf;
	buf <<"CCD-"<< boost::format("%08d") % information;
	if (buf.str().size() >= 8) {
		owner()->version = buf.str();

	}
	GetControllerCardModel(car);
	printf("CardModel: %s \n", car);
	//info=car;
	//cout<<"info "<< info<<endl;
}



void RealIKonDriver::setCooler(int status) {

	if (status==0)
		CoolerOFF();
	else
		CoolerON();
}




}
