#include "DoubleBufferHandler.h"

#include <iostream>
#include <sstream>

#include "BufferInformation.h"

CDoubleBufferHandler::CDoubleBufferHandler()
: buffer_read_(nullptr)
, buffer_write_(nullptr)
, new_data(false)
{
}

CDoubleBufferHandler::~CDoubleBufferHandler()
{
}

void CDoubleBufferHandler::PushBuffer(BGAPI2::Buffer *buffer) {
    std::lock_guard<std::mutex> lock(buffer_exchange_lock_);
    if( !buffer_write_ ) {
        buffer_write_ = buffer;
        new_data = true;
    }
    else {
        buffer->QueueBuffer();
    }
}

BGAPI2::Buffer* CDoubleBufferHandler::PullBuffer() {
    std::lock_guard<std::mutex> lock(buffer_exchange_lock_);
    if (buffer_write_ ) {
        if (buffer_read_) {
            buffer_read_->QueueBuffer();
        }
        buffer_read_ = buffer_write_;
        buffer_write_ = nullptr;
        new_data = false;
    }
    return buffer_read_;
}

void CDoubleBufferHandler::FreeBuffer(BGAPI2::Buffer *buffer) {
    std::lock_guard<std::mutex> lock(buffer_exchange_lock_);
    if (buffer_read_ && buffer_write_) {
        buffer_read_->QueueBuffer();
        buffer_read_ = nullptr;
    }
}

void CDoubleBufferHandler::Init() {
    if (buffer_write_) {
        buffer_write_->QueueBuffer();
        buffer_write_ = nullptr;
    }
    if (buffer_read_) {
        buffer_read_->QueueBuffer();
        buffer_read_ = nullptr;
    }
    new_data = false;
}

bool CDoubleBufferHandler::HasNewData() {
    return new_data;
}

CDoubleBufferHandler::CDoubleBufferHandler(const CDoubleBufferHandler &instance) {
    buffer_read_ = instance.buffer_read_;
    buffer_write_ = instance.buffer_write_;
}

CDoubleBufferHandler & CDoubleBufferHandler::operator =(const CDoubleBufferHandler &instance) {
    buffer_read_ = instance.buffer_read_;
    buffer_write_ = instance.buffer_write_;
    return *this;
}
