/*
 * Nomad Instrument Control Software
 *
 * Copyright 2011 Institut Laue-Langevin
 *
 * Licensed under the EUPL, Version 1.1 only (the "License");
 * You may not use this work except in compliance with the Licence.
 * You may obtain a copy of the Licence at:
 *
 * http://joinup.ec.europa.eu/software/page/eupl
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the Licence is distributed on an "AS IS" basis,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the Licence for the specific language governing permissions and
 * limitations under the Licence.
 */

#include "SimulatedGenericCamDriver.h"

#include <stdlib.h>
#include "tiffio.h"
#include "GenericCamDef.h"
#include "GenericCamDriver.h"

namespace genericcam {

/*
 * Constructor
 */
SimulatedGenericCamDriver::SimulatedGenericCamDriver(GenericCamDriver* owner) :
				GenericCamState(owner) {
	/* Empty */
}

/*
 * Destructor
 */
SimulatedGenericCamDriver::~SimulatedGenericCamDriver() {
	/* Empty */
}

void SimulatedGenericCamDriver::init() {
}

/*
 * clearvme/daugther/GenericCam
 */
void SimulatedGenericCamDriver::clear() {
	// Read command
	owner()->time = 0;
	owner()->counts = 0;
	owner()->counts2 = 0;
	owner()->counts3 = 0;
}

/*
 * writeParam
 */
void SimulatedGenericCamDriver::writeParam() {
}
//
///*
// * regroup
// */
//void SimulatedGenericCamDriver::regroup() {
//}

/*
 * synchroniseRead
 */
void SimulatedGenericCamDriver::synchroniseRead() {
	read();
	owner()->m_SynchroniseReadTerminated = true;
}

/*
 * read
 */
void SimulatedGenericCamDriver::read() {
	int32 size = owner()->dataSize();
	int32 *tab = owner()->data();
	float64 sum = 0;
	boost::filesystem::path file = owner()->fileSimu();
	TIFF* tif = TIFFOpen(file.string().c_str(), "r");
	if (tif) {
	  	int32 dircount = 0;
	  	while (dircount < owner()->indexSimu()) {
	  		if (TIFFReadDirectory(tif) <= 0) break;
	  		++dircount;
	  	};
	  	uint32 w, h;
	  	int32 npixels;
	  	tdata_t buf;

	  	TIFFGetField(tif, TIFFTAG_IMAGEWIDTH, &w);
	  	TIFFGetField(tif, TIFFTAG_IMAGELENGTH, &h);
	  	npixels = w * h;
    	if (npixels == size) {
    		tmsize_t scanlinesize = TIFFScanlineSize(tif);
    		buf = _TIFFmalloc(scanlinesize);
			if (buf != NULL) {
				uint16* data;
				for (uint32 row = 0; row < h; row++) {
					TIFFReadScanline(tif, buf, row, 0);
					data = (uint16*) buf;
					for(uint32 i = 0; i < w;++i) {
						tab[row + i * h] = data[i];
						sum = sum + data[i];
					}
				}
				owner()->sum = sum;
				owner()->data.sendEvent();
				_TIFFfree(buf);
			}
    	}
	  	TIFFClose(tif);
	}

	// Read command
//	for (int32 i = 0; i < size; i++) {
//		tab[i] = i;
//		sum = sum + tab[i];
//	}
}

/*
 * pause 
 */
void SimulatedGenericCamDriver::pause() {
}

/*
 * continue 
 */
void SimulatedGenericCamDriver::resume() {
}

/*
 * start
 */
void SimulatedGenericCamDriver::start() {
	owner()->time = owner()->time.setpoint();
}

/*
 * stop
 */
void SimulatedGenericCamDriver::stop() {
}

/*
 * readStatus
 */
void SimulatedGenericCamDriver::readStatus() {
	owner()->startActivated = false;
	// Status command
	owner()->computeGateStatus(genericcam::READOUT_COMPLETE);
	owner()->computeAcquisitionStatus(0);
	owner()->computeDetectorStatus(0);
	if (owner()->m_SynchroniseReadActivated == true) {
		if (owner()->m_SynchroniseReadTerminated == true) {
			owner()->m_SynchroniseReadActivated = false;
			owner()->m_SynchroniseReadTerminated = false;
			owner()->commandProgression = PROGRESSION_END_DEVICE_CONTAINER;
		}
	} else {
		owner()->commandProgression = PROGRESSION_END_DEVICE_CONTAINER;
	}
}

/*
 * readInfos
 */
void SimulatedGenericCamDriver::readInfos() {
}

}
