/*
 * Nomad Instrument Control Software
 *
 * Copyright 2011 Institut Laue-Langevin
 *
 * Licensed under the EUPL, Version 1.1 only (the "License");
 * You may not use this work except in compliance with the Licence.
 * You may obtain a copy of the Licence at:
 *
 * http://joinup.ec.europa.eu/software/page/eupl
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the Licence is distributed on an "AS IS" basis,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the Licence for the specific language governing permissions and
 * limitations under the Licence.
 */

#include "RealCfdDriver.h"
#include "CfdDriver.h"
#include "CfdDef.h"

#include "Utilities/Calculations.h"
#include "Utilities/Counter.h"

using namespace driver;

namespace cfd {

/*
 * Constructor
 */
RealCfdDriver::RealCfdDriver(CfdDriver* owner) :
		CfdState(owner) {

}

/*
 * Destructor
 */
RealCfdDriver::~RealCfdDriver() {

}

/*
 * init
 */
void RealCfdDriver::init() {
//	//Reset all parameters --> to be done
//	// init A32 base adress of the ioxos board.
//	uint32 aData;
//	// enable VME Slave
//	aData = ENABLE_VME_SLAVE;
//	owner()->write(ENABLE_VME_SLAVE_REGISTER, Vme::LONGWORD_WIDTH, 1, (void*) &aData);
//
//	// Init A32 VME BA
//	aData = owner()->address() >> 16;
//	owner()->write(A32_INIT_BA_REGISTER, Vme::LONGWORD_WIDTH, 1, (void*) &aData);
//
//	// Enable A32 VME BA
//	aData = ENABLE_A32_BA;
//	owner()->write(ENABLE_A32_BA_REGISTER, Vme::LONGWORD_WIDTH, 1, (void*) &aData);
//
//	// MMU Pointer
//	aData = MMU_PAGE_ZERO;
//	owner()->write(MMU_PAGE_ZERO_REGISTER, Vme::LONGWORD_WIDTH, 1, (void*) &aData);
//
//	// Descriptor for SUSER
//	aData = DESCRIPTOR_FOR_SUSER;
//	owner()->write(DESCRIPTOR_FOR_SUSER_REGISTER, Vme::LONGWORD_WIDTH, 1, (void*) &aData);
//	// same address, data = 3
//	aData = 3;
//	owner()->write(DESCRIPTOR_FOR_SUSER_REGISTER, Vme::LONGWORD_WIDTH, 1, (void*) &aData);
//	// same address, data = 0 (first write)
//	aData = 0;
//	owner()->write(DESCRIPTOR_FOR_SUSER_REGISTER, Vme::LONGWORD_WIDTH, 1, (void*) &aData);
//	// same address, data = 0 (second write)
//	aData = 0;
//	owner()->write(DESCRIPTOR_FOR_SUSER_REGISTER, Vme::LONGWORD_WIDTH, 1, (void*) &aData);
//
//	// MMU Pointer cf Thierry
//	aData = 8;
//	owner()->write(MMU_PAGE_ZERO_REGISTER, Vme::LONGWORD_WIDTH, 1, (void*) &aData);
//
//	// Descriptor for SUSER cf Thierry
//	aData = DESCRIPTOR_FOR_SUSER_2;
//	owner()->write(DESCRIPTOR_FOR_SUSER_REGISTER, Vme::LONGWORD_WIDTH, 1, (void*) &aData);
//	// same address, data = 3
//	aData = 3;
//	owner()->write(DESCRIPTOR_FOR_SUSER_REGISTER, Vme::LONGWORD_WIDTH, 1, (void*) &aData);
//	// same address, data = 0 (first write)
//	aData = 0;
//	owner()->write(DESCRIPTOR_FOR_SUSER_REGISTER, Vme::LONGWORD_WIDTH, 1, (void*) &aData);
//	// same address, data = 0 (second write)
//	aData = 0;
//	owner()->write(DESCRIPTOR_FOR_SUSER_REGISTER, Vme::LONGWORD_WIDTH, 1, (void*) &aData);
//
//	// Enabling the VMEP2 SerDes
//	// Enable PLL Clock Generation.
//	aData = ENABLE_PLL_CLOCK;
//	owner()->write(ENABLE_PLL_CLOCK_REGISTER, Vme::LONGWORD_WIDTH, 1, (void*) &aData);
//
//	// Enable SerDes.
//	aData = ENABLE_SERDES_VMEP2;
//	owner()->write(ENABLE_SERDES_VMEP2_REGISTER, Vme::LONGWORD_WIDTH, 1, (void*) &aData);

	uint32 value = 1;
	owner()->write(GLOBAL_PARAMETER + 0x14, Vme::LONGWORD_WIDTH, 1, (void*) &value);

	readInfos();
}

/*
 * writeParam
 */
void RealCfdDriver::writeParam() {

	uint32 value = 1;

	//owner()->write(GLOBAL_PARAMETER + 0x14, Vme::LONGWORD_WIDTH, 1, (void*) &value);

	//value = 0x1000;
	// Internal list mode
	//owner()->write(GLOBAL_PARAMETER + 0xC0, Vme::LONGWORD_WIDTH, 1, (void*) &value);

	value = owner()->dppAcqMode();
	//cout << "dppAcqMode = " << value << endl;
	owner()->write(GLOBAL_PARAMETER + 0xC8, Vme::LONGWORD_WIDTH, 1, (void*) &value);

	value = owner()->nbChannels();
	//cout << "nbChannels = " << value << endl;
	owner()->write(GLOBAL_PARAMETER + 0xCC, Vme::LONGWORD_WIDTH, 1, (void*) &value);

	value = (int32) ceil((float) owner()->nbChannels() / 8.);
	//cout << "nbChannels 2 = " << value << endl;
	owner()->write(GLOBAL_PARAMETER + 0xD0, Vme::LONGWORD_WIDTH, 1, (void*) &value);

	value = 1000;
	owner()->write(GLOBAL_PARAMETER + 0xD8, Vme::LONGWORD_WIDTH, 1, (void*) &value);

	//cout << "scopeChannel = " << owner()->scopeChannel() << endl;
	value = owner()->scopeChannel() & 0xFF;
	//cout << "scopeChannel = " << value << endl;
	for (int32 i = 0; i < owner()->nbChannels(); ++i) {
		value += (owner()->pulsePolarity.get(i) & 0x1) << (16 + i);
	}
	owner()->write(GLOBAL_PARAMETER + 0xD4, Vme::LONGWORD_WIDTH, 1, (void*) &value);

	value = owner()->delay() & 0xFFFF;
	//cout << "delay = " << value << endl;
	value += 3 << 16;
	owner()->write(GLOBAL_PARAMETER + 0xF0, Vme::LONGWORD_WIDTH, 1, (void*) &value);

	value = owner()->attenuation();
	//cout << "attenuation = " << value << endl;
	owner()->write(GLOBAL_PARAMETER + 0xF4, Vme::LONGWORD_WIDTH, 1, (void*) &value);

	for (int32 i = 0; i < owner()->nbChannels(); ++i) {
		value = owner()->trapezoidRise.get(i) & 0xFF;
		value += (owner()->trapezoidFlat.get(i) & 0xFF) << 8;
		value += (owner()->decayTime.get(i) & 0xFF) << 16;
		value += (owner()->peaking.get(i) & 0xFF) << 24;
		owner()->write(GLOBAL_PARAMETER + 0x240 + i * 0x4, Vme::LONGWORD_WIDTH, 1, (void*) &value);

		value = owner()->bitShift.get(i);
		owner()->write(GLOBAL_PARAMETER + 0x280 + i * 0x4, Vme::LONGWORD_WIDTH, 1, (void*) &value);
	}

	for (int32 i = 0; i < (int32) ceil((float) owner()->nbChannels() / 8.); ++i) {	// OSCILLOSCOPE MODE FOR CFD v1751
		// HACK EMILIO
		value = (i << 30) + 0x8000;
		//cout << "attenuation = " << value << endl;
		owner()->write(GLOBAL_PARAMETER + 0xE8, Vme::LONGWORD_WIDTH, 1, (void*) &value);

		value = 0x00070110;
		//cout << "attenuation = " << value << endl;
		owner()->write(GLOBAL_PARAMETER + 0xF8, Vme::LONGWORD_WIDTH, 1, (void*) &value);

		value = 1;
		//cout << "attenuation = " << value << endl;
		owner()->write(GLOBAL_PARAMETER + 0x18, Vme::LONGWORD_WIDTH, 1, (void*) &value);
	}



}

/*
 * readInfos
 */
void RealCfdDriver::readInfos() {

}

///*
// * pow2roundup
// */
//int32 RealCfdDriver::pow2roundup(int32 x) {
//
//	if (x < 0)
//		return 0;
//	--x;
//	x |= x >> 1;
//	x |= x >> 2;
//	x |= x >> 4;
//	x |= x >> 8;
//	x |= x >> 16;
//	return x + 1;
//}

}
