/*
 * Nomad Instrument Control Software
 *
 * Copyright 2011 Institut Laue-Langevin
 *
 * Licensed under the EUPL, Version 1.1 only (the "License");
 * You may not use this work except in compliance with the Licence.
 * You may obtain a copy of the Licence at:
 *
 * http://joinup.ec.europa.eu/software/page/eupl
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the Licence is distributed on an "AS IS" basis,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the Licence for the specific language governing permissions and
 * limitations under the Licence.
 */

#include "RealPslDriver.h"
#include "PslDef.h"
#include "PslDriver.h"
#include <boost/format.hpp>
#include "Utilities/Counter.h"

#include <cstdio>
#include "CImg.h"

using namespace cimg_library;
using namespace std;
using namespace boost;

namespace psl {

/*
 * Constructor
 */
RealPslDriver::RealPslDriver(PslDriver* owner) :
										PslState(owner) {
	/* Empty */
}

/*
 * Destructor
 */
RealPslDriver::~RealPslDriver() {
}

/*
 * init
 */
void RealPslDriver::init() {
	readInfos();
}

/*
 * clear
 */
void RealPslDriver::clear() {
}

/*
 * writeParam
 */
void RealPslDriver::writeParam() {
	ostringstream buf_conf;
	std::size_t found ;


	string err;

	buf_conf << "SetAcquisitionMode;"<< boost::format("%03d") %PslDriver::MODE[owner()->acqmode()];
	err = writeAndRead(buf_conf.str(), TERMINATOR);
	buf_conf.str("");
	buf_conf.clear();
	found = err.find("True");
	if (found==std::string::npos)
	{
		cerr << "error SetAcquisitionMode " << endl;
	}
	buf_conf << "SetAutoSave;1";
	err = writeAndRead(buf_conf.str(), TERMINATOR);
	buf_conf.str("");
	buf_conf.clear();
	found = err.find("True");
	if (found==std::string::npos)
	{
		cerr << "error SetAutoSave " << endl;
	}

	buf_conf << "SetShowAverage;0";
	err = writeAndRead(buf_conf.str(), TERMINATOR);
	buf_conf.str("");
	buf_conf.clear();
	found = err.find("True");
	if (found==std::string::npos)
	{
		cerr << "error SetShowAverage " << endl;
	}

	buf_conf << "SetBrightPixel;" <<  boost::format("%03d") % owner()->bpixel();
	err = writeAndRead(buf_conf.str(), TERMINATOR);
	buf_conf.str("");
	buf_conf.clear();
	found = err.find("True");
	if (found==std::string::npos)
	{
		cerr << "error SetBrightPixel " << endl;
	}

	buf_conf << "SetIntensifierGain;"<< boost::format("%02d") %  owner()->gain();
	err = writeAndRead(buf_conf.str(), TERMINATOR);
	buf_conf.str("");
	buf_conf.clear();
	found = err.find("True");
	if (found==std::string::npos)
	{
		cerr << "error SetIntensifierGain " << endl;
	}
	if (owner()->acqmode()==0){
		owner()->nbSlicesAcq=1;
	}
	owner()->nbFrame=owner()->nbSlicesAcq();
//	buf_conf << "SetFrameNumber;"<<  boost::format("%02d") % owner()->nbSlicesAcq();
	buf_conf << "SetFrameNumber;"<<  owner()->nbSlicesAcq();
	err = writeAndRead(buf_conf.str(), TERMINATOR);
	buf_conf.str("");
	buf_conf.clear();
	found = err.find("True");
	if (found==std::string::npos)
	{
		cerr << "error SetIntensifierGain " << endl;
	}

}

/*
 * synchroniseRead
 */
void RealPslDriver::synchroniseRead() {
	string err;
	int32 i;
	std::size_t found ;

	cout << "m_TimeCounter" <<m_TimeCounter.getTime()<< endl;
	//sleep(5);
	//	owner()->setTimeoutValue( owner()->time.setpoint());

	//	do {
	err = writeAndRead("WaitForImage", TERMINATOR);
	cout << "Waiting For Image" << endl;
	//	    sleep(1);
	//	} while ((err != "True") && (i++ <10 ));
	//	if (err != "True") {
	//		cerr << "error acquisition " << endl;
	//	} else {
	//	stop();
	//sleep(2);

	string buf;
	owner()->connect();
	Counter c1;
	cout << "<-- GetImage" << endl;
	owner()->write("GetImage", TERMINATOR);


	if (owner()->acqmode()==1){
		buf = owner()->read(23);
	}else{
		buf = owner()->read(21);
	}
	//	owner()->disconnect();
	cout << "--> buf " << buf << endl;
	typedef tokenizer<char_separator<char> > tokenizer;
	char_separator<char> sep(";");
	tokenizer tok1(buf, sep);
	tokenizer::iterator iter = tok1.begin();
	if (iter != tok1.end()) {
		cout << " iter 1 : " << *iter << endl;
		string format = lexical_cast<string>(*iter);
		++iter;
		if (iter != tok1.end()) {
			cout << " iter 2 : " << *iter << endl;
			int32 xsize = lexical_cast<int32>(*iter);
			++iter;
			if (iter != tok1.end()) {
				cout << " iter 3 : " << *iter << endl;
				int32 ysize = lexical_cast<int32>(*iter);;
				++iter;
				if (iter != tok1.end()) {
					cout << " iter 4 : " << *iter << endl;
					int32 size = xsize * ysize;//lexical_cast<int32>(*iter);
					//sleep(5);
					found = format.find("I16");
					if (found != std::string::npos)
					{
						cerr << "datas type I16" << endl;
						string datas = owner()->read(size * 2);
						uint16* tmp = (uint16*) (datas.c_str());
						cout << "datas size = " << datas.size() << endl;
						int32* data = owner()->data();
						int32 dataSize = owner()->dataSize();
						float64 sum = 0;
						if ((data != NULL) && (size == dataSize)) {
							CImg<uint16> img(tmp, xsize, ysize);
							img.permute_axes("yxzc");
							img.mirror("x");
							memcpy(tmp, img.data(), xsize * ysize * sizeof(uint16));
							for (int32 x = 0; x < xsize; ++x) {
								for (int32 y = 0; y < ysize; ++y) {
									data[x + y * xsize] = tmp[x + y * xsize];
									sum += (float64) data[y + x * ysize];
								}
							}
							owner()->data.sendEvent();
						} else {
							cerr << "error mask " << endl;

						}
					}else{
						cerr << "datas type F" << endl;
						owner()->setTimeoutValue(10);
						string datas = owner()->read(size * 4);

						uint32* tmp = (uint32*) (datas.c_str());
						cout << "datas size = " << datas.size() << endl;
						int32* data = owner()->data();
						int32 dataSize = owner()->dataSize();
						float64 sum = 0;
						if ((data != NULL) && (size == dataSize)) {
							CImg<uint32> img(tmp, xsize, ysize);
							img.permute_axes("yxzc");
							img.mirror("x");
							memcpy(tmp, img.data(), xsize * ysize * sizeof(uint32));
							for (int32 x = 0; x < xsize; ++x) {
								for (int32 y = 0; y < ysize; ++y) {
									data[x + y * xsize] = tmp[x + y * xsize];
									sum += (float64) data[y + x * ysize];
								}
							}
							owner()->data.sendEvent();
						} else {
							cerr << "error mask " << endl;

						}
					}
				}
			}
		}
	}
	owner()->disconnect();
	cout << "read time:" << c1.getTime() << endl;

	cout << "owner()->m_SynchroniseReadTerminated = true" << endl;
	owner()->m_SynchroniseReadTerminated = true;
}

/*
 * read
 */
void RealPslDriver::read() {
}

/*
 * start
 */
void RealPslDriver::start() {
	ostringstream buf_conf;
	float64 exp_time = owner()->time.setpoint();
	std::size_t found;
	string err;

			if (owner()->acqmode()==3){
				owner()->time.setpoint=exp_time*owner()->nbSlicesAcq();
			}

	buf_conf << "SetRecordPath;"<<  "z:"<<owner()->fileDirectory();
	err = writeAndRead(buf_conf.str(), TERMINATOR);
	buf_conf.str("");
	buf_conf.clear();
	found = err.find("True");
	if (found==std::string::npos)
	{
		cerr << "error SetRecordPath; " << endl;
	}
	buf_conf << "SetRecordName;"<<   owner()->fileName();
	err = writeAndRead(buf_conf.str(), TERMINATOR);
	buf_conf.str("");
	buf_conf.clear();
	found = err.find("True");
	if (found==std::string::npos)
	{
		cerr << "error SetRecordName; " << endl;
	}
	buf_conf << "SetRecordNumber;"<< owner()->numor();
	err = writeAndRead(buf_conf.str(), TERMINATOR);
	buf_conf.str("");
	buf_conf.clear();
	found = err.find("True");
	if (found==std::string::npos)
	{
		cerr << "error SetRecordNumber;; " << endl;
	}

	buf_conf << "SetRecordTag;"<< owner()->header();
	err = writeAndRead(buf_conf.str(), TERMINATOR);
	buf_conf.str("");
	buf_conf.clear();
	found = err.find("True");
	if (found==std::string::npos)
	{
		cerr << "error SetRecordTag; " << endl;
	}

	cout << "start" << endl;
	cout << "exp_time:" << exp_time << endl;
	buf_conf << "SetExposure;[" << exp_time << ",\"Second\"]";

	err = writeAndRead(buf_conf.str(), TERMINATOR);

	err = writeAndRead("Snap", TERMINATOR);



	const char* home = getenv("HOME");
	string homePath;
	if (home != 0) {
		homePath = string(home)+"/lastfile.txt";
	}

	ofstream file(homePath, ios::out);
	file << owner()->fileName() << owner()->numor()<< endl;

	file.close();
	found = err.find("True");
	cout << "found  Snap " <<found<< "  " << std::string::npos<<endl;
	if (found==std::string::npos)
	{
		cerr << "error acquisition " << endl;
		owner()->startActivated = false;
		owner()->commandProgression = PROGRESSION_END_DEVICE_CONTAINER;
	}

	m_TimeCounter.clear();

}

/*
 * resume
 */
void RealPslDriver::resume() {
}

/*
 * pause
 */
void RealPslDriver::pause() {

}

/*
 * stop
 */
void RealPslDriver::stop() {
	ostringstream buf_cmd2;
	writeAndRead("Stop", TERMINATOR);
	int32 status = 0;
	status |= END;
	owner()->computeGateStatus(status);
	owner()->computeAcquisitionStatus(0);
	owner()->computeDetectorStatus(0);
	owner()->startActivated = false;
	owner()->commandProgression = PROGRESSION_END_DEVICE_CONTAINER;
}

/*
 * readStatus
 */
void RealPslDriver::readStatus() {

	float64 actualtime = m_TimeCounter.getTime();
	owner()->time = actualtime;

	int32 status = 0;
	if (owner()->startActivated() == true) {
		if (actualtime > owner()->time.setpoint()+0.5) {
			status |= END;
			owner()->startActivated = false;
			owner()->commandProgression = PROGRESSION_END_DEVICE_CONTAINER;
		} else {
			status |= COUNTING;
			owner()->calculateProgression();
		}
	} else {
		if (owner()->m_SynchroniseReadActivated == true) {
			if (owner()->m_SynchroniseReadTerminated == true) {
				owner()->m_SynchroniseReadActivated = false;
				owner()->m_SynchroniseReadTerminated = false;
				owner()->commandProgression = PROGRESSION_END_DEVICE_CONTAINER;
			}
		} else {
			owner()->commandProgression = PROGRESSION_END_DEVICE_CONTAINER;
		}
	}
	owner()->computeGateStatus(status);
	owner()->computeAcquisitionStatus(0);
	owner()->computeDetectorStatus(0);
}

/*
 * readInfos
 */
void RealPslDriver::readInfos() {
	string info = writeAndRead("GetVersion", TERMINATOR);
	owner()->version = info;
}

/*
 * writeAndRead
 */
std::string RealPslDriver::writeAndRead(const std::string& command,
		const std::string& terminator) {
	string buf;
	owner()->connect();
	owner()->setTimeoutValue(6000);
	cout << "<-- " << command << endl;
	owner()->write(command, terminator);
//	usleep(10000);
	cout << "<-- OK " << command << endl;
	buf = owner()->read();
	cout << "--> " << buf << endl;
	owner()->disconnect();
	usleep(100000);
	return buf;

}
std::string RealPslDriver::writeAndRead(const std::string& command,
		const std::string& terminator,const uint32 nbchar) {
	string buf;
	owner()->connect();
	//	owner()->setTimeoutValue(5);
	cout << "<-- " << command << endl;

	owner()->write(command, terminator);
	cout << "<-- OK nb char " << command << endl;
	buf = owner()->read(nbchar);
	cout << "--> " << buf << endl;
	owner()->disconnect();
	return buf;

}
}
