/*
 * Nomad Instrument Control Software
 *
 * Copyright 2011 Institut Laue-Langevin
 *
 * Licensed under the EUPL, Version 1.1 only (the "License");
 * You may not use this work except in compliance with the Licence.
 * You may obtain a copy of the Licence at:
 *
 * http://joinup.ec.europa.eu/software/page/eupl
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the Licence is distributed on an "AS IS" basis,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the Licence for the specific language governing permissions and
 * limitations under the Licence.
 */

#include <cmath>

#include "StereoFluxgateController.h"
#include "controllers/common/family/Families.h"

using namespace std;

namespace stereo_fluxgate {

const string StereoFluxgateController::TYPE = "stereo_fluxgate_controller";

/*
 * Constructor
 */
StereoFluxgateController::StereoFluxgateController(const string& name) :ExperimentController(name){


	setFamily(family::SAMPLE_ENVIRONMENT);

	// Init property
	nbfluxgate.init(this, SAVE, "nbfluxgate");
	ichannel.init(this, NOSAVE, "ichannel");
	Bx.init(this,NOSAVE,"Bx");
	By.init(this,NOSAVE,"By");
	Bz.init(this,NOSAVE,"Bz");
	Btotal.init(this,NOSAVE,"Btotal");
	m_Driver.init(this, "driver");

	registerFunction(TYPE);


}

StereoFluxgateController::StereoFluxgateController(const StereoFluxgateController& controller) :
	ExperimentController(controller){

	nbfluxgate.copy(this, controller.nbfluxgate);
	ichannel.copy(this, controller.ichannel);
	Bx.copy(this, controller.Bx);
	By.copy(this, controller.By);
	Bz.copy(this, controller.Bz);
	Btotal.copy(this, controller.Btotal);



	m_Driver.copy(controller.m_Driver);

}

/*
 * Destructor
 */
StereoFluxgateController::~StereoFluxgateController() {
	// Empty
}


void StereoFluxgateController::postConfiguration() {
	registerRefresher(nbfluxgate,&StereoFluxgateController::refreshNbFluxgateProperty, this);
	//nbfluxgate=m_Driver->nbfluxgate();
	registerUpdater(m_Driver->nbfluxgate, &StereoFluxgateController::updateNbFluxgate, this);
	registerUpdater(m_Driver->Bx, &StereoFluxgateController::updateBx, this);
	registerUpdater(m_Driver->By, &StereoFluxgateController::updateBy, this);
	registerUpdater(m_Driver->Bz, &StereoFluxgateController::updateBz, this);

}

void StereoFluxgateController::refreshNbFluxgateProperty(int32 value) throw (CannotSetValue) {

	Bx.resize(value);
	By.resize(value);
	Bz.resize(value);
	Btotal.resize(value);
	ichannel.resize(value);
	for(int i=0;i<value;i++){
		Bx.set(i,m_Driver->Bx(i));
		//cout << "Bx=" << Bx(i) << endl;
		By.set(i,m_Driver->By(i));
		Bz.set(i,m_Driver->Bz(i));
		Btotal.set(i,sqrt(Bx(i)*Bx(i)+By(i)*By(i)+Bz(i)*Bz(i)));
		ichannel.set(i,i);
	}
}

void StereoFluxgateController::updateNbFluxgate(){
	nbfluxgate=m_Driver->nbfluxgate();
}

void StereoFluxgateController::updateBx(int32 index){
	Bx.set(index,m_Driver->Bx(index));
	Btotal.set(index,sqrt(Bx(index)*Bx(index)+By(index)*By(index)+Bz(index)*Bz(index)));
}
void StereoFluxgateController::updateBy(int32 index){
	By.set(index,m_Driver->By(index));
	Btotal.set(index,sqrt(Bx(index)*Bx(index)+By(index)*By(index)+Bz(index)*Bz(index)));
}
void StereoFluxgateController::updateBz(int32 index){
	Bz.set(index,m_Driver->Bz(index));
	Btotal.set(index,sqrt(Bx(index)*Bx(index)+By(index)*By(index)+Bz(index)*Bz(index)));
}



}

